@@ -1,3 +1,4 @@
+/* -*- c-basic-offset: 2 -*- */
 #include <config.h>
 #include "xwrits.h"
 #include <stdlib.h>
@@ -38,6 +39,7 @@ Port **ports;
 
 fd_set x_socket_set;
 int max_x_socket;
+XErrorHandler old_x_error_handler;
 
 int check_idle;
 struct timeval idle_time;
@@ -55,6 +57,8 @@ struct timeval quota_allotment;
 int max_cheats;
 static int allow_cheats;
 
+static int run_once;
+
 int verbose;
 
 static int force_mono = 0;
@@ -65,7 +69,7 @@ static void
 short_usage(void)
 {
   fprintf(stderr, "Usage: xwrits [-display DISPLAY] [typetime=TIME] [breaktime=TIME] [OPTION]...\n\
-Try `xwrits --help' for more information.\n");
+Try 'xwrits --help' for more information.\n");
   exit(1);
 }
 
@@ -74,15 +78,15 @@ static void
 usage(void)
 {
   printf("\
-`Xwrits' reminds you to take wrist breaks, which should help you prevent or\n\
+'Xwrits' reminds you to take wrist breaks, which should help you prevent or\n\
 manage a repetitive stress injury. It runs on X.\n\
 \n\
 Usage: xwrits [-display DISPLAY] [typetime=TIME] [breaktime=TIME] [OPTION]...\n\
 \n\
 All options may be abbreviated to their unique prefixes. The three forms\n\
-`OPTION', `--OPTION', and `+OPTION' are equivalent. Options listed as\n\
-`+OPTION' can be on or off; they are normally off, and you can turn them off\n\
-explicitly with `-OPTION'.\n\
+'OPTION', '--OPTION', and '+OPTION' are equivalent. Options listed as\n\
+'+OPTION' can be on or off; they are normally off, and you can turn them off\n\
+explicitly with '-OPTION'.\n\
 \n\
 General options:\n\
   --display DISPLAY   Monitor the X display DISPLAY. You can monitor more than\n\
@@ -106,17 +110,20 @@ Break characteristics:\n\
   +cheat[=NUM]        Allow NUM keystrokes before cancelling a break.\n\
   canceltime=TIME, ct=TIME  Allow typing for TIME after a break is cancelled\n\
                       (default 10 minutes).\n\
+  +once               Quit after the warning window is clicked on. Useful for\n\
+                      setting alarms.\n\
 \n");
   printf("\
 Appearance:\n\
   after=TIME          Change behavior if warning window is ignored for TIME.\n\
-                      Options following `after' give new behavior.\n\
+                      Options following 'after' give new behavior.\n\
   +beep               Beep when the warning window appears.\n\
   +breakclock         Show how much time remains during the break.\n\
   +clock              Show how long you have ignored the warning window.\n\
   +finger             Be rude.\n\
-  +finger=CULTURE     Be rude according to CULTURE. Choices: `american',\n\
-                      `korean' (synonyms `japanese', `russian'), `german'.\n\
+  +finger=CULTURE     Be rude according to CULTURE. Choices: 'american',\n\
+                      'korean' (synonyms 'japanese', 'russian'), 'german',\n\
+                      or the name of any GIF file.\n\
   flashtime=RATE      Flash the warning window at RATE (default 2 sec).\n\
   +iconified          Warning windows appear as icons.\n\
   lock-picture=GIF-FILE, lp=GIF-FILE     Show GIF animation on lock window.\n\
@@ -124,7 +131,7 @@ Appearance:\n\
   +mono               Use monochrome pictures.\n\
   +multiply=PERIOD    Make a new warning window every PERIOD.\n\
   +noiconify          Don't let anyone iconify the warning window.\n\
-  ready-picture=GIF-FILE, okp=GIF-FILE   Show GIF animation on the `OK' window.\n\
+  ready-picture=GIF-FILE, okp=GIF-FILE   Show GIF animation on the 'OK' window.\n\
   rest-picture=GIF-FILE, rp=GIF-FILE     Show GIF animation on resting window.\n\
   title=TITLE         Set xwrits window title to TITLE.\n\
   +top                Keep the warning windows on top of the window stack.\n\
@@ -269,11 +276,14 @@ notify_peers_rest(void)
 void
 mark_xwrits_window(Port *port, Window w)
 {
-  int scrap = port->permanent_hand->w;
+  uint32_t scrap = port->permanent_hand->w;
   assert(scrap);
   XChangeProperty(port->display, w, port->xwrits_window_atom,
 		  XA_INTEGER, 32, PropModeReplace,
 		  (unsigned char *)&scrap, 1);
+  XChangeProperty(port->display, w, port->wm_client_leader_atom,
+		  XA_WINDOW, 32, PropModeReplace,
+		  (unsigned char *)&scrap, 1);
 }
 
 Window
@@ -382,8 +392,14 @@ default_x_processing(XEvent *e)
     if (h) {
       if (e->xclient.message_type == port->wm_protocols_atom
 	  && (Atom)(e->xclient.data.l[0]) == port->wm_delete_window_atom) {
+	/* window manager delete */
 	e->type = Xw_DeleteWindow;
 	destroy_hand(h);
+      } else if (e->xclient.message_type == port->wm_protocols_atom
+		 && (Atom)(e->xclient.data.l[0]) == port->net_wm_ping_atom) {
+	/* window manager pinging us to make sure we're still alive */
+	e->xclient.window = port->root_window;
+	XSendEvent(port->display, port->root_window, True, 0, e);
       } else if (e->xclient.message_type == port->xwrits_break_atom
 		 && e->xclient.data.l[0] != 0)
 	e->type = Xw_TakeBreak;
@@ -407,9 +423,7 @@ default_x_processing(XEvent *e)
     /* We must unschedule any IdleSelect events for fear of selecting input on
        a destroyed window. There is a race condition here because X
        communication is asynchronous. */
-    port = find_port(display, e->xdestroywindow.window);
-    unschedule_data(A_IDLE_SELECT, (void *)port,
-		    (void *)e->xdestroywindow.window);
+    unschedule_data(A_IDLE_SELECT, (void *) e->xdestroywindow.window);
     if (verbose)
 	fprintf(stderr, "Window 0x%x: destroyed\n", (unsigned)e->xdestroywindow.window);
     break;
@@ -449,7 +463,7 @@ strtointerval(char *s, char **stores, struct timeval *iv)
     sec += strtod(s+1, &s);
     ok = 1;
   } else if (*s == ':' && s[1] == 0)
-    /* accept `MINUTES:' */
+    /* accept 'MINUTES:' */
     s++;
 
   /* return */
@@ -513,7 +527,9 @@ optparse(char *arg, char *option, int unique, char *format, ...)
     if (*arg++ != *opt++) return 0;
     unique--;
   }
-  if (unique > 0) return 0;
+  /* 4.Sep.2002 - Prevent inappropriate identification of an option (could
+     mistake --helpcdsainhfds for --help). */
+  if (unique > 0 || (*arg && *arg != '=')) return 0;
   
   if (*arg == '=') {
     arg++;
@@ -578,7 +594,7 @@ slideshow_text_append_built_in(Options *o, char *built_in)
   
   sprintf(s, "%s%s&%s", (ls ? o->slideshow_text : ""),
 	  (ls ? ";" : ""), built_in);
-  sprintf(is, "%s%s&%sicon", (lis ? o->icon_slideshow_text : ""),
+  sprintf(is, "%s%s*%sicon", (lis ? o->icon_slideshow_text : ""),
 	  (lis ? ";" : ""), built_in);
   
   if (!o->prev || o->prev->slideshow_text != o->slideshow_text)
@@ -620,10 +636,10 @@ parse_options(int pargc, char **pargv)
       ;
     else if (optparse(s, "breaktime", 1, "st", &o->break_time)) {
       if (o->prev && !breaktime_warn_context) {
-	warning("meaning of `breaktime' following `after' has changed");
-	message("(You can specify `breaktime' multiple times, to lengthen a break");
+	warning("meaning of 'breaktime' following 'after' has changed");
+	message("(You can specify 'breaktime' multiple times, to lengthen a break");
 	message("if you ignore xwrits for a while. In previous versions, there was");
-	message("one global `breaktime'.)");
+	message("one global 'breaktime'.)");
 	breaktime_warn_context = 1;
       } else
 	breaktime_warn_context = 1;
@@ -639,7 +655,12 @@ parse_options(int pargc, char **pargv)
       ;
     else if (optparse(s, "cheat", 2, "tI", &max_cheats))
       allow_cheats = optparse_yesno;
-    else if (optparse(s, "clock", 1, "t"))
+    else if (optparse(s, "once", 2, "tI", &run_once)) {
+	if (optparse_yesno && run_once < 0)
+	    run_once = 1;
+	else if (!optparse_yesno)
+	    run_once = 0;
+    } else if (optparse(s, "clock", 1, "t"))
       o->clock = optparse_yesno;
     
     else if (optparse(s, "display", 1, "ss", &arg)) {
@@ -727,7 +748,7 @@ parse_options(int pargc, char **pargv)
     else if (optparse(s, "version", 1, "s")) {
       printf("LCDF Xwrits %s\n", VERSION);
       printf("\
-Copyright (C) 1994-2002 Eddie Kohler\n\
+Copyright (C) 1994-2006 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
@@ -869,6 +890,9 @@ default_settings(void)
   xwSETTIME(onormal.next_delay, 15 * SEC_PER_MIN, 0);
   onormal.next = 0;
   onormal.prev = 0;
+
+  /* how many times to run? */
+  run_once = -1;
 }
 
 
@@ -895,9 +919,15 @@ initialize_slave_port(Port *port)
     port->white = m->white;
     port->font = m->font;
     port->gfx = m->gfx;
-    port->wm_delete_window_atom = m->wm_delete_window_atom;
     port->wm_protocols_atom = m->wm_protocols_atom;
+    port->wm_delete_window_atom = m->wm_delete_window_atom;
+    port->wm_client_leader_atom = m->wm_client_leader_atom;
     port->mwm_hints_atom = m->mwm_hints_atom;
+    port->net_wm_ping_atom = m->net_wm_ping_atom;
+    port->net_wm_desktop_atom = m->net_wm_desktop_atom;
+    port->net_wm_window_type_atom = m->net_wm_window_type_atom;
+    port->net_wm_window_type_utility_atom = m->net_wm_window_type_utility_atom;
+    port->net_wm_pid_atom = m->net_wm_pid_atom;
     port->xwrits_window_atom = m->xwrits_window_atom;
     port->xwrits_notify_peer_atom = m->xwrits_notify_peer_atom;
     port->xwrits_break_atom = m->xwrits_break_atom;
@@ -1006,9 +1036,15 @@ initialize_port(int portno)
     (display, screen_number, port->visual, port->depth, port->colormap);
 
   /* set atoms */
-  port->wm_delete_window_atom = XInternAtom(display, "WM_DELETE_WINDOW", False);
   port->wm_protocols_atom = XInternAtom(display, "WM_PROTOCOLS", False);
+  port->wm_delete_window_atom = XInternAtom(display, "WM_DELETE_WINDOW", False);
+  port->wm_client_leader_atom = XInternAtom(display, "WM_CLIENT_LEADER", False);
   port->mwm_hints_atom = XInternAtom(display, "_MOTIF_WM_HINTS", False);
+  port->net_wm_ping_atom = XInternAtom(display, "_NET_WM_PING", False);
+  port->net_wm_desktop_atom = XInternAtom(display, "_NET_WM_DESKTOP", False);
+  port->net_wm_window_type_atom = XInternAtom(display, "_NET_WM_WINDOW_TYPE", False);
+  port->net_wm_window_type_utility_atom = XInternAtom(display, "_NET_WM_WINDOW_TYPE_UTILITY", False);
+  port->net_wm_pid_atom = XInternAtom(display, "_NET_WM_PID", False);
   port->xwrits_window_atom = XInternAtom(display, "XWRITS_WINDOW", False);
   port->xwrits_notify_peer_atom = XInternAtom(display, "XWRITS_NOTIFY_PEER", False);
   port->xwrits_break_atom = XInternAtom(display, "XWRITS_BREAK", False);
@@ -1107,6 +1143,8 @@ main_loop(void)
 	    break;
 
 	  case ST_REST:
+	    if (run_once > 0 && run_once == 1)
+		exit(0);
 	    was_lock = 0;
 	    tran = rest();
 	    assert(tran == TRAN_AWAKE || tran == TRAN_CANCEL || tran == TRAN_FAIL);
@@ -1129,6 +1167,8 @@ main_loop(void)
 	    break;
 
 	  case ST_AWAKE:
+	    if (run_once > 0 && --run_once == 0)
+		exit(0);
 	    ready();
 	    unmap_all();
 	    s = ST_NORMAL_WAIT;
@@ -1163,7 +1203,7 @@ main(int argc, char *argv[])
   for (i = 0; i < orig_nports; i++) {
       Display *display = XOpenDisplay(ports[i]->display_name);
       if (!display)
-	  error("can't open display `%s'", ports[i]->display_name);
+	  error("can't open display '%s'", ports[i]->display_name);
       ports[i]->display = display;
       if (!multiscreen)
 	  ports[i]->screen_number = DefaultScreen(display);
@@ -1261,7 +1301,7 @@ main(int argc, char *argv[])
   
   /* watch keystrokes on all windows */
   xwGETTIME(now);
-  XSetErrorHandler(x_error_handler);
+  old_x_error_handler = XSetErrorHandler(x_error_handler);
   for (i = 0; i < nports; i++)
       if (ports[i]->master == ports[i])
 	  watch_keystrokes(ports[i], ports[i]->root_window, &now);
