@@ -24,21 +24,24 @@ static unsigned long key_press_selected_count;
 int
 x_error_handler(Display *d, XErrorEvent *error)
 {
-  if (error->error_code != BadWindow && error->error_code != BadDrawable) {
-    char buffer[BUFSIZ];
-    XGetErrorText(d, error->error_code, buffer, BUFSIZ);
-    fprintf(stderr, "X Error of failed request: %s\n", buffer);
-    abort();
-  }
-  
-  /* Maybe someone created a window then destroyed it immediately!
-     I don't think there's any way of working around this. */
-  if (error->error_code == BadWindow) {
-    Window window = (Window)error->resourceid;
-    unschedule_data(A_IDLE_SELECT, (void *)find_port(error->display, window),
-		    (void *)window);
-  }
-  return 0;
+    if (error->error_code != BadWindow && error->error_code != BadDrawable) {
+	char buffer[BUFSIZ];
+	/* avoid recursive calls to this function, in case XGetErrorText
+	   returns an error! Problem reported by Mike Swieton
+	   <swietonm@student.gvsu.edu> */
+	XSetErrorHandler(old_x_error_handler);	
+	XGetErrorText(d, error->error_code, buffer, BUFSIZ);
+	fprintf(stderr, "X Error of failed request: %s\n", buffer);
+	abort();
+    }
+
+    /* Maybe someone created a window then destroyed it immediately!
+       I don't think there's any way of working around this. */
+    if (error->error_code == BadWindow) {
+	Window window = (Window)error->resourceid;
+	unschedule_data(A_IDLE_SELECT, (void *) window);
+    }
+    return 0;
 }
 
 
@@ -65,7 +68,7 @@ watch_keystrokes(Port *port, Window w, const struct timeval *now)
   /* This code ensures that at least register_keystrokes_delay elapses before
      we listen for KeyPress events on the window. We want to wait so we can
      make sure the window selects them first. */
-  a = new_alarm_data(A_IDLE_SELECT, (void *)port, (void *)w);
+  a = new_alarm_data(A_IDLE_SELECT, (void *)w, (void *)port);
   xwADDTIME(a->timer, *now, register_keystrokes_delay);
   schedule(a);
   
@@ -79,7 +82,7 @@ void
 register_keystrokes(Port *port, Window w)
 {
   /* Before I only selected KeyPress if someone else had selected KeyPress on
-     the window (indicated by the `or' of all_event_masks and
+     the window (indicated by the 'or' of all_event_masks and
      do_not_propagate_mask). That seems obviously wrong! What if they only
      selected KeyRelease? What if they selected KeyPress later? So I've
      removed it. Hopefully this won't cause any strange behavior. */
@@ -173,15 +176,13 @@ schedule(Alarm *newalarm)
 }
 
 void
-unschedule_data(int actions, void *data1, void *data2)
+unschedule_data(int actions, void *data1)
 {
   Alarm *a = alarm_sentinel.next;
   while (a != &alarm_sentinel) {
     Alarm *n = a->next;
     
-    if ((a->action & actions) != 0
-	&& (a->data1 == data1 || data1 == 0)
-	&& (a->data2 == data2 || data2 == 0)) {
+    if ((a->action & actions) != 0 && (a->data1 == data1 || data1 == 0)) {
       a->prev->next = n;
       n->prev = a->prev;
       xfree(a);
@@ -254,7 +255,7 @@ loopmaster(Alarmloopfunc alarm_looper, Xloopfunc x_looper)
 	break;
 	
        case A_IDLE_SELECT:
-	register_keystrokes((Port *)a->data1, (Window)a->data2);
+	register_keystrokes((Port *)a->data2, (Window)a->data1);
 	break;
 	
        case A_MOUSE: {
