@@ -9,8 +9,8 @@
 #ifndef FD_SET
 #include <sys/select.h>
 #endif
-#include "gif.h"
-#include "gifx.h"
+#include <lcdfgif/gif.h>
+#include <lcdfgif/gifx.h>
 
 typedef struct Port Port;
 typedef struct Options Options;
@@ -65,9 +65,15 @@ struct Port {
   
   Gif_XContext *gfx;		/* GIF X context */
   
-  Atom wm_delete_window_atom;	/* atoms for window manager communication */
-  Atom wm_protocols_atom;
+  Atom wm_protocols_atom;	/* atoms for window manager communication */
+  Atom wm_delete_window_atom;
+  Atom wm_client_leader_atom;
   Atom mwm_hints_atom;
+  Atom net_wm_ping_atom;
+  Atom net_wm_desktop_atom;
+  Atom net_wm_window_type_atom;
+  Atom net_wm_window_type_utility_atom;
+  Atom net_wm_pid_atom;
   Atom xwrits_window_atom;	/* atoms for communication with other xwrits */
   Atom xwrits_notify_peer_atom;
   Atom xwrits_break_atom;
@@ -107,6 +113,7 @@ Window check_xwrits_window(Port *, Window);
 #define Xw_TakeBreak		(LASTEvent + 101)
 int default_x_processing(XEvent *);
 int x_error_handler(Display *, XErrorEvent *);
+extern XErrorHandler old_x_error_handler;
 
 void add_peer(Port *, Window);
 void notify_peers_rest(void);
@@ -207,8 +214,8 @@ void destroy_alarm(Alarm *);
 
 void init_scheduler(void);
 void schedule(Alarm *);
-#define unschedule(i)	unschedule_data((i), 0, 0)
-void unschedule_data(int, void *, void *);
+#define unschedule(i)	unschedule_data((i), 0)
+void unschedule_data(int, void *);
 
 int loopmaster(Alarmloopfunc, Xloopfunc);
 
@@ -239,6 +246,7 @@ struct Hand {
   
   unsigned is_icon: 1;
   unsigned mapped: 1;
+  unsigned withdrawn: 1;
   unsigned configured: 1;
   unsigned obscured: 1;
   unsigned clock: 1;
@@ -258,6 +266,7 @@ void destroy_hand(Hand *);
 Hand *find_one_hand(Port *, int mapped);
 
 Hand *window_to_hand(Port *, Window, int allow_icon);
+void hand_map_raised(Hand *);
 
 void draw_slide(Hand *);
 
